local function SafeFreeSlot(...)
	for _,slot in ipairs({...})
		if not rawget(_G, slot) freeslot(slot) end -- Ensure we don't accidentally overlap existing freeslots
	end
end
SafeFreeSlot("sfx_sready", "sfx_slspgo", "sfx_salspd", "sfx_lspdst", "sfx_lspdlo")
sfxinfo[sfx_sready].caption = '\x85"READYYY!"\x80'
sfxinfo[sfx_slspgo].caption = '\x85"GO!!"\x80'
sfxinfo[sfx_salspd].caption = 'Lightspeed Dash'
sfxinfo[sfx_lspdst].caption = 'Lightspeed Dash Start!'
sfxinfo[sfx_lspdlo].caption = '/'


rawset(_G, "cv_kombilspd", CV_RegisterVar({
	name = "k_lightspeeddash",
	defaultvalue = "On",
	flags = CV_SAVE|CV_SHOWMODIF,
	PossibleValue = {On = 1, Off = 0},
}))


addHook("PlayerSpawn", function(player)
	if (player.realmo.skin == "kombi") and not srb2p
		player.lightspeedhold = 0
		player.lightspeeddashable = 0
		player.timervar2 = 0
	end
end)

local function SearchForGrabbables(player)
	local closest = nil
	searchBlockmap("objects", function(refmobj, mo)
		if mo.health <= 0 then -- We don't wanna grab something that's dead, that's kinda gross!
			return
        end
		
		if (mo.flags & MF_SPECIAL) then -- We're probably trying to grab the portal or somewhere similar
			return
        end

		if not P_CheckSight(player.realmo, mo) then -- Out of sight
			return
        end

		if closest and P_AproxDistance(P_AproxDistance(player.realmo.x-mo.x, player.realmo.y-mo.y),
			player.realmo.z-mo.z) > P_AproxDistance(P_AproxDistance(player.realmo.x-closest.x,
			player.realmo.y-closest.y), player.realmo.z-closest.z) then
			return
        end
		
		closest = mo
	end, player.realmo, player.realmo.x-64*FRACUNIT, player.realmo.x+64*FRACUNIT, player.realmo.y-64*FRACUNIT, player.realmo.y+64*FRACUNIT)
	return closest or false
end

addHook("SpinSpecial", function(player)
	if (player.realmo.skin == "trez") and not P_IsObjectOnGround(player.realmo)
		/*if SearchForGrabbables(player)
		print("Grab!",SearchForGrabbables(player).type)
		else*/
		player.mo.state = S_PLAY_SLAM
		player.slamstate = 2
		--end
	end
	if (player.realmo.skin == "kombi") and not srb2p
	    and (P_IsObjectOnGround(player.realmo))
		and not (player.realmo.state == S_PLAY_PAIN)
		and not (player.realmo.state == S_PLAY_DEAD)
		and not (player.realmo.state == S_PLAY_DRWN)
		and not (player.realmo.state == S_PLAY_SPINDASH)
		and not player.nospindash
		if (player.realmo.state == S_PLAY_ROLL)
			player.nospindash = true
			player.realmo.state = S_PLAY_STND
			player.pflags = $&~PF_SPINNING
			player.pflags = $&~PF_STARTDASH
			return true
		end
		if player.lightspeeddashable > 0 and CV_FindVar("k_lightspeeddash").string == "On" then return true end
		player.pflags = $|PF_SPINNING|PF_STARTDASH
		player.charability2 = 0 -- Don't even fight the default CHARABILITY2 here. We aren't gonna win.
		player.realmo.state = S_PLAY_SPINDASH
		player.drawangle = player.realmo.angle
		player.lightspeedhold = leveltime
		S_StartSound(player.realmo, sfx_spndsh)
		S_StartSound(player.realmo, sfx_spndsh)
		if CV_FindVar("k_lightspeeddash").string == "On"
			S_StartSound(player.realmo, sfx_salspd)
		end
		return true
	end
end)


addHook("PlayerThink", function(player)
	if player.lightspeedhold == nil
		player.lightspeedhold = 99999*TICRATE
	end
	if player.lightspeeddashable == nil
		player.lightspeeddashable = 0
	end
	if player.timervar2 == nil
		player.timervar2 = 99999*TICRATE
	end
	if (player.realmo.state == S_PLAY_PAIN) and player.lightspeeddashable == 2
		player.lightspeeddashable = 0
	end
	if (player.lightspeedhold - leveltime) <= -100 and player.realmo.state == S_PLAY_SPINDASH
		if CV_FindVar("k_lightspeeddash").string == "On"
			player.realmo.colorized = true
			S_StopSoundByID(player.realmo, sfx_salspd)
			player.charability2 = CA2_SPINDASH
			S_StartSound(player.realmo, sfx_sready)
			player.lightspeeddashable = 1
			player.realmo.state = S_PLAY_STND
			player.pflags = $&~PF_SPINNING
			player.pflags = $&~PF_STARTDASH
		end
	end
	if CV_FindVar("k_lightspeeddash").string == "Off" and player.fcolorize == true
		player.fcolorize = false
	end
end)


addHook("PlayerThink", function(player)
	if not (player.cmd.buttons & BT_SPIN)
		player.nospindash = false
	end
	if (player.pflags & PF_STARTDASH)
	and player.realmo.skin == "kombi"
	and not (player.cmd.buttons & BT_SPIN)
		if player.lightspeeddashable > 0 and CV_FindVar("k_lightspeeddash").string == "On" then return true end
		//player.charability2 = CA2_SPINDASH
		player.realmo.state = S_PLAY_ROLL
		player.pflags = $&~PF_STARTDASH
		P_Thrust(player.realmo, player.drawangle, min(player.mindash+(leveltime-player.lightspeedhold)*FRACUNIT,player.maxdash))
		S_StopSoundByID(player.realmo, sfx_salspd)
		S_StartSound(player.realmo, sfx_zoom)
	end
end)


-- Others


/* addHook("PlayerThink", function(player)
	if (player.realmo.skin == "kombi") and (player.realmo.state == S_PLAY_SPINDASH) and not srb2p
		player.realmo.momz = 0
        player.realmo.momy = 0
		player.realmo.momx = 0
	end	
end) */

-- Commence Lightspeed Dash Section

local lspdrange = 360*FRACUNIT
local lspdspeed = 60*FRACUNIT
local lspdskin = "kombi"

local function P_RingNearby(player) -- Is a ring in range?
    local closest
	closest = NULL

	searchBlockmap("objects", function(refmobj, mo)

		if mo.health <= 0 then -- Not a valid ring
			return
        end

		if not mo.type == 107 then
			return
        end

		if P_AproxDistance(P_AproxDistance(player.realmo.x-mo.x, player.realmo.y-mo.y),
			player.realmo.z-mo.z) > lspdrange then -- Out of range
			return
        end

		if not P_CheckSight(player.realmo, mo) then -- Out of sight
			return
        end

		if closest and P_AproxDistance(P_AproxDistance(player.realmo.x-mo.x, player.realmo.y-mo.y),
			player.realmo.z-mo.z) > P_AproxDistance(P_AproxDistance(player.realmo.x-closest.x,
			player.realmo.y-closest.y), player.realmo.z-closest.z) then
			return
        end

		-- Found a target
		closest = mo
    end, player.realmo, player.realmo.x-lspdrange, player.realmo.x+lspdrange, player.realmo.y-lspdrange, player.realmo.y+lspdrange)

	if closest then
		print(closest.type)
		return true
    end

	return false
end

local function P_LightDash(source, enemy) -- Home in on your target

	if not source.tracer then
		return -- Nothing to home in on!
    end

	-- adjust direction
	local dest = source.tracer

	if not dest then
		return
    end
	
	-- change angle
	source.player.angle = R_PointToAngle2(source.x, source.y, enemy.x, enemy.y)

	-- change slope
	local dist = P_AproxDistance(P_AproxDistance(dest.x - source.x, dest.y - source.y), dest.z - source.z);

	if dist < 1 then
		dist = 1
    end

	source.momx = FixedMul(FixedDiv(dest.x - source.x, dist), lspdspeed)
	source.momy = FixedMul(FixedDiv(dest.y - source.y, dist), lspdspeed)
	source.momz = FixedMul(FixedDiv(dest.z - source.z, dist), lspdspeed)
end

local function P_LookForRings(player)

    local found
    player.realmo.target = NULL
    player.realmo.tracer = NULL

	searchBlockmap("objects", function(refmobj, mo)

		if mo.health <= 0 then -- Not a valid ring
			return
        end

		if not mo.type == 107 then
			return
        end

		if P_AproxDistance(P_AproxDistance(player.realmo.x-mo.x, player.realmo.y-mo.y),
			player.realmo.z-mo.z) > lspdrange then -- Out of range
			return
        end

		if not P_CheckSight(player.realmo, mo) then -- Out of sight
			return
        end

		if player.realmo.target and P_AproxDistance(P_AproxDistance(player.realmo.x-mo.x, player.realmo.y-mo.y), player.realmo.z-mo.z) > 
			P_AproxDistance(P_AproxDistance(player.realmo.x-player.realmo.target.x, player.realmo.y-player.realmo.target.y), player.realmo.z-player.realmo.target.z) then
			return
        end

		-- Found a target
		if mo.type == 107 then
			found = true
			player.realmo.target = mo
			player.realmo.tracer = mo
        end
    end, player.realmo, player.realmo.x-lspdrange, player.realmo.x+lspdrange, player.realmo.y-lspdrange, player.realmo.y+lspdrange)

	if found and player.realmo.tracer.type == 107 then
		player.lightdash = TICRATE
		P_ResetPlayer(player)
		player.realmo.state = S_PLAY_FALL
		P_ResetScore(player)
		P_LightDash(player.realmo, player.realmo.tracer)
		return
	else
		player.pflags = $|PF_JUMPED
		player.pflags = $&~PF_THOKKED
		player.lightspeeddashable = 0
		player.lightdash = 0
		player.attackdown = false
    end
	player.realmo.momx = FixedDiv(player.realmo.momx,2*FRACUNIT)
	player.realmo.momy = FixedDiv(player.realmo.momy,2*FRACUNIT)
end

addHook("PlayerThink", function(player)
	-- aqua: checks i added
	if (lspdskin and player.realmo.skin ~= lspdskin)
	or player.exiting or P_PlayerInPain(player)
	-- or player.pflags & PF_GLIDING or player.pflags & PF_SLIDING or player.pflags & PF_SHIELDABILITY
	or (player.climbing or player.powers[pw_carry]) then
		player.lightdash = nil
		return
	end

	-- aqua: i removed the player.lightdashallowed variable for optimization purposes.
		-- because, before, to make that unused variable work, i would need to search for rings every frame,
		-- whilst now, i only have to search for rings when you press the light dash button.
		-- however, if you intend on porting the HUD element for the light dash from final demo, you will need to reimplement it.
	if player.lightspeeddashable >= 1
		if not (player.cmd.buttons & BT_SPIN)
			if player.lightspeedhold >= leveltime
				P_InstaThrust(player.realmo, player.realmo.angle, 60*FRACUNIT)
			end
			if player.lightspeeddashable == 1
				player.realmo.colorized = false
				S_StartSound(player.realmo, sfx_lspdst)
				S_StartSound(player.realmo, sfx_slspgo)
				player.lightspeedhold = leveltime + 7
				player.lightdash = 0
				player.lightspeeddashable = 2
			end
			if player.lightspeedhold == leveltime and not player.attackdown
				player.attackdown = true
				if P_RingNearby(player)
					player.lightdash = TICRATE
					player.lightspeedhold = leveltime
				else
					player.realmo.momy = 0*FRACUNIT
					player.realmo.momx = 0*FRACUNIT
					player.lightspeeddashable = 0
					player.attackdown = false
				end
			end
		end
	else
		if not player.kombiboosting
			S_StopSoundByID(player.realmo, sfx_lspdst)
			S_StopSoundByID(player.realmo, sfx_lspdlo)
		end
		player.attackdown = false
    end

	if not (player.lightdash == nil)
		if player.lightdash > 0
			player.lightdash = $ - 1
			P_LookForRings(player)
			P_SpawnGhostMobj(player.realmo)
			player.powers[pw_flashing] = 2
			if not S_SoundPlaying(player.mo, sfx_lspdst) and not S_SoundPlaying(player.mo, sfx_lspdlo)
				S_StartSound(player.realmo, sfx_lspdlo)
			end
		end
    end
end)